<?php declare(strict_types=1);

namespace Convo\Proto\Pckg\Elements;

class ProductSearch extends \Convo\Core\Workflow\AbstractWorkflowComponent implements \Convo\Core\Workflow\IConversationElement
{
    private $_json_path;
    private $_search;
    private $_result;
    private $_page_size;
    private $_page_no;
    
    public function __construct($properties) {
        
        parent::__construct( $properties);
        $this->_json_path = $properties['json_path'];
        $this->_search = $properties['search'];
        $this->_result = $properties['result'];
        $this->_page_size = $properties['page_size'];
        $this->_page_no = $properties['page_no'];
        
    }
    
    public function read( \Convo\Core\Workflow\IConvoRequest $request, \Convo\Core\Workflow\IConvoResponse $response)
    {
        $this->_logger->debug( 'Raw json path ['.$this->_json_path.']');
        $this->_logger->debug( 'Raw search ['.$this->_search.']');
        $this->_logger->debug( 'Raw result ['.$this->_result.']');
        $this->_logger->debug( 'Raw page size ['.$this->_page_size.']');
        $this->_logger->debug( 'Raw page number ['.$this->_page_no.']');
        
        $json_path	=   $this->evaluateString( $this->_json_path."products.json");
        $search	=   $this->evaluateString( $this->_search);
        $result	=   $this->evaluateString( $this->_result);
        $page_size = $this->evaluateString( $this->_page_size);
        $page_no = $this->evaluateString( $this->_page_no);
        
        $this->_logger->debug( 'Using json path ['.$json_path.']');
        $this->_logger->debug( 'search ['.$search.']');
        $this->_logger->debug( 'var to store ['.$result.']');
        $this->_logger->debug('Page size ['.$page_size.']');
        $this->_logger->debug('Page number ['.$page_no.']');
        
        $json_file = file_get_contents($json_path);
        $json = json_decode($json_file, true);
        
        if (($json != $json_file) && $json)
        {
            $this->_logger->debug('JSON is valid');
        }
        else
        {
            $this->_logger->debug('JSON is invalid');
        }
        
        $params = $this->getService()->getServiceParams(\Convo\Core\Params\IServiceParamsScope::SCOPE_TYPE_SESSION);
        
        if ( empty($page_size))
        {
            $page_size = 3;
        }
        
        if ( empty($page_no))
        {
            $page_no = 1;
        }
        
        $num_products = 0;
        $has_next_page = false;
        $has_prev_page = false;
        
        if ( empty( $search))
        {
            $num_products = count( $json);
            $data_products = [];
            $this->_logger->debug('Number of products ['.print_r( $num_products, true).']');

            if ( $num_products > $page_size)
            {
                $has_next_page = true;
            }
            if ( $num_products < $page_size)
            {
                $page_size = $num_products;
            }
            if ( $page_no > 1)
            {
                $has_prev_page = true;
            }
            
            $offset = ( $page_no - 1) * $page_size;
            $end = $offset + $page_size;
            
            for ( $i = $offset; $i < $end; $i++)
            {
                if ( !empty($json[$i]))
                {
                    array_push( $data_products, $json[$i]);
                }
            }
            
            $params->setServiceParam($result, [
                'count' => $num_products,
                'data' => $data_products,
                'has_next_page' => $has_next_page,
                'has_prev_page' => $has_prev_page
            ]);
            
            $this->_logger->debug('Found products ['.print_r( $data_products, true).']');
        }

        if ( !empty( $search))
        {
            $length = count( $json);
            $found_products = [];
            $data = [];
            $max_pages = 0;
            
            for ( $i = 0; $i < $length; $i++)
            {
                if ( stripos( $json[$i]['title'], $search) !== false)
                {
                    array_push( $found_products, $json[$i]);
                }
            }
                $num_products = count( $found_products);
                $max_pages = ceil( $num_products / $page_size);
                
                if ( $num_products > $page_size)
                {
                    $has_next_page = true;
                }
                
                if ( $num_products < $page_size)
                {
                    $page_size = $num_products;
                }
                if ( $page_no > 1)
                {
                    $has_prev_page = true;
                }
                
                $offset = ( $page_no - 1) * $page_size;
                $end = $offset + $page_size;
                
                for ( $j = $offset; $j < $end; $j++)
                {
                    if ( !empty($found_products[$j]))
                    {
                        array_push( $data, $found_products[$j]);
                    }
                }
                if ( $max_pages == $page_no)
                {
                    $has_next_page = false;
                }
                
                $params->setServiceParam($result, [
                    'count' => $num_products,
                    'data' => $data,
                    'has_next_page' => $has_next_page,
                    'has_prev_page' => $has_prev_page
                ]);

            $this->_logger->debug('Found products ['.print_r( $data, true).']');
        }
        
    }
    
    
    // UTIL
    public function __toString()
    {
        return parent::__toString().'['.$this->_json_path.']';
    }
    
}

?>
