<?php

declare(strict_types=1);

namespace Convo\Proto;

use Convo\Core\DataItemNotFoundException;
use Psr\Http\Server\RequestHandlerInterface;

class ServiceUsersRestHandler implements RequestHandlerInterface
{
    /**
     * @var \Convo\Proto\ServiceUserDao
     */
    private $_serviceUsersManager;
    
    /**
     * @var \Convo\Core\Util\IHttpFactory
     */
    private $_httpFactory;
    
    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $_logger;
    
    public function __construct( $httpFactory, $logger, $serviceUserDao)
    {
    	$this->_logger				= 	$logger;
    	$this->_httpFactory			= 	$httpFactory;
        $this->_serviceUsersManager	=	$serviceUserDao;
    }

    public function handle(\Psr\Http\Message\ServerRequestInterface $request): \Psr\Http\Message\ResponseInterface
    {
    	$info			=	new \Convo\Core\Rest\RequestInfo( $request);
    	
    	if ( $info->get() && $info->route( 'users'))
    	{
    		return $this->_handleUsersGet( $request);
    	}
    	
    	if ( $info->get() && $route = $info->route( 'users/{userId}'))
    	{
    		return $this->_handleUsersPathUserIdGet( $request, $route->get( 'userId'));
    	}
    	
    	if ( $info->put() && $route = $info->route( 'users/{userId}'))
    	{
    		return $this->_handleUsersPut( $request, $route->get( 'userId'));
    	}
    	
    	throw new \Convo\Core\Rest\NotFoundException( 'Could not map ['.$info.']');
    }

    private function _handleUsersGet(\Psr\Http\Message\ServerREquestInterface $request)
    {
        $data = $this->_serviceUsersManager->getUsers();

        $marshalled = [];

        foreach ($data as $user) {
            $marshalled[] = [
                'userId' => $user['userId'],
                'firstName' => $user['firstName'],
                'lastName' => $user['lastName'],
                'email' => $user['email']
            ];
        }

        return $this->_httpFactory->buildResponse($marshalled);
    }

    private function _handleUsersPathUserIdGet(\Psr\Http\Message\ServerRequestInterface $request, $userId)
    {
        try {
            $user = $this->_serviceUsersManager->getUserById($userId);

            return $this->_httpFactory->buildResponse([
                'userId' => $user['userId'],
                'firstName' => $user['firstName'],
                'lastName' => $user['lastName'],
                'email' => $user['email']
            ]);
        } catch (DataItemNotFoundException $e) {
            $this->_logger->error($e->getMessage());
            return $this->_httpFactory->buildResponse(['message' => "No user with ID [$userId]"], 404);
        }
    }

    private function _handleUsersPut(\Psr\Http\Message\ServerRequestInterface $request, $userId)
    {
    	$json = $request->getParsedBody();

        $userId = $this->_serviceUsersManager->updateUser($userId, $json);

        return $this->_httpFactory->buildResponse([]);
    }

    // UTIL
    public function __toString()
    {
        return get_class($this) . '[]';
    }
}
